/** 
 Agent Class: Abstract base class for all agents
Users creating their own agents will have to override the methods given here

  */
import java.security.*;

public abstract class Agent 
{
	// get agent name
	public abstract String getAgentName();

	// get the origin host ip address
	public abstract String getAgentOrigin();

	// get the previous host ip address
	public abstract String getAgentPreviousLocation();

	// get the organization from where the agent origined
	public abstract String getAgentOrganization();

	// get the next host in the agent host list to visit
	public abstract String getNextHostAddress();
	
	// tells the next host for dispatch and removes it from the list
	public abstract String getNextHostAddressForDispatch();

	// tell the current location of the agent
	public abstract void tellLocation();

	// tell the data that the agent is currently holding
	public abstract void tellData();

	// show the results of the transactions
	public abstract void showResults();

	// get the Message digest
	public abstract byte[] giveMessageDigest();

	// get the public key
	public abstract PublicKey givePublicKey();

	// get the signed object
	public abstract byte[] giveSignature();

	// sign the agent
	public abstract void signAgent(PublicKey key, byte digest[], byte sign[]);


	// the MAIN thread of operation
	public abstract void run();
}