/** 
class : AgentServerGUI

This class represents the GUI for the agent Server
Accepts user requests to spawn new agents and also shown the incoming and ready to go agent lists
*/

import java.awt.swing.*;
import java.awt.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.swing.border.*;
import java.awt.swing.text.*;
import java.util.*;




public class AgentServerGUI extends AppFrame implements ActionListener
{
	OutAgentListBox readyList;
	InAgentListBox incomingList;
	ResultAgentListBox resultList;
	Container c;
	AgentInitializer agentInitializer;
	AgentServer agentServer;
	Agent toDispatch;
	SelectedAgent selectedAgent;
	DefaultStyledDocument doc;
	SimpleAttributeSet bigStyle;
	JButton b1;
	JButton b2;
	JButton b3;
	JButton b4;
	String selectedAgentName=null;

	int agentSelectedToSpawn=1;


	// boolean values for all the settings
	boolean askBeforeDispatch=false;
	boolean showReceiveEvents=false;
	boolean isAgentSelected=false;
	boolean askBeforeAccepting=false;
	boolean acceptAgent=true;
	String operation=null;

	// Constructor
	public AgentServerGUI(String operation)
	{
		super(400,500);
		this.operation=operation;
		c=getContentPane();
		c.setLayout(new BorderLayout());
		setTitle("Agent Server 1.0");
		addMenuComponents();
		addToolBarComponents();	
		addMessageWindow();
		addListComponents();
	
		show();
	}

	/* add all the menu components and set the menubar	*/
	private void addMenuComponents()
	{
		JMenuBar menuBar=new JMenuBar();

		// Agent Menu 
		JMenu agentMenu=new JMenu("Agent Settings");

		// Agent Menu Items
		JCheckBoxMenuItem a1=new JCheckBoxMenuItem("Show Receiver Events");
		JCheckBoxMenuItem a2=new JCheckBoxMenuItem("Ask Before Dispatch");
		JMenuItem a3=new JMenuItem("Change Personal Settings");
		JCheckBoxMenuItem a4=new JCheckBoxMenuItem("Ask Before Starting Agent");
		
		// Server Menu
		JMenu serverMenu=new JMenu("Server");

		// Server Menu Items
		JMenuItem s1=new JMenuItem("Start");
		JMenuItem s2=new JMenuItem("Stop");


		// add menu items
		agentMenu.add(a1);
		agentMenu.add(a2);
		agentMenu.add(a4);
		agentMenu.addSeparator();
		agentMenu.add(a3);
		serverMenu.add(s1);
		serverMenu.add(s2);

		// add action listeners
		a1.addActionListener(this);
		a2.addActionListener(this);
		a3.addActionListener(this);
		a4.addActionListener(this);
		s1.addActionListener(this);
		s2.addActionListener(this);

		// add action commands
		a1.setActionCommand("showReceiverEvents");
		a4.setActionCommand("askBeforeAccepting");
		a2.setActionCommand("askBeforeDispatch");
		a3.setActionCommand("ChangeSetting");
		s1.setActionCommand("Start");
		s2.setActionCommand("Stop");


		// AGENTS
		JMenu currentAgents=new JMenu("Current Agents");
		ButtonGroup rbg = new ButtonGroup();

		// Radio button menus for the agents
		JRadioButtonMenuItem c1=new JRadioButtonMenuItem("MyAgent");
		c1.addActionListener(this);
		JRadioButtonMenuItem c2=new JRadioButtonMenuItem("InfoAgent");
		c2.addActionListener(this);
		JRadioButtonMenuItem c3=new JRadioButtonMenuItem("TransferAgent");
		c3.addActionListener(this);
		JRadioButtonMenuItem c4=new JRadioButtonMenuItem("BillAgent");
		c4.addActionListener(this);



		// add the components
		if (operation.equals("user"))
		{
			rbg.add(c1);
			rbg.add(c2);
			rbg.add(c3);
			currentAgents.add(c1);
			currentAgents.add(c2);
			currentAgents.add(c3);
		}
		else if (operation.equals("telecom"))
		{
			rbg.add(c4);
			currentAgents.add(c4);
		}

		// Add Menu to MenuBar
		menuBar.add(serverMenu);
		menuBar.add(agentMenu);
		menuBar.add(currentAgents);

		setJMenuBar(menuBar);
	}

	/* add all the buttons to the tool bar and set the toolbar	*/
	private void addToolBarComponents()
	{
		JToolBar toolBar=new JToolBar();
		toolBar.setLayout(new FlowLayout());
		toolBar.setBackground(Color.black);
		toolBar.setForeground(Color.white);
		toolBar.setFloatable(false);

		// The image icons
		ImageIcon icon1=new ImageIcon("spawnon.gif");
		ImageIcon icon2=new ImageIcon("dispatchon.gif");
		ImageIcon icon3=new ImageIcon("killon.gif");
		ImageIcon icon4=new ImageIcon("forward.gif");
		ImageIcon icon5=new ImageIcon("logo.gif");

		// The font
		Font myFont=new Font("Serif",Font.BOLD|Font.ITALIC,10);
		JLabel label=new JLabel(" a ");
		label.setIcon(icon5);
		label.setFont(myFont);

		b1=new JButton(icon1);
		b1.setBackground (SystemColor.control);
		b1.setMargin(new Insets(0,0,0,0));
		b1.setActionCommand("Spawn");
		b1.addActionListener(this);
		b1.setToolTipText("Spawn a new agent");

		b2=new JButton(icon2);
		b2.setEnabled(false);
		b2.setBackground (SystemColor.control);
		b2.setMargin(new Insets(0,0,0,0));
		b2.setActionCommand("Dispatch");
		b2.addActionListener(this);
		b2.setToolTipText("Dispatch selected agent");

		b3=new JButton(icon3);
		b3.setEnabled(false);
		b3.setBackground (SystemColor.control);
		b3.setMargin(new Insets(0,0,0,0));
		b3.setActionCommand("Kill");
		b3.addActionListener(this);
		b3.setToolTipText("Kill selected agent");

		b4=new JButton(icon4);
		b4.setEnabled(false);
		b4.setBackground (SystemColor.control);
		b4.setMargin(new Insets(0,0,0,0));
		b4.setActionCommand("Forward");
		b4.addActionListener(this);
		b4.setToolTipText("Change User Settings");

		toolBar.add(b1);
		toolBar.add(b2);
		toolBar.add(b3);
		toolBar.add(b4);
		toolBar.add(label);

		c.add(toolBar,BorderLayout.NORTH);
	}

	/* add the 3 list boxes for the agents	*/
	private void addListComponents()
	{
		readyList=new OutAgentListBox(this);
		incomingList=new InAgentListBox(this);
		resultList=new ResultAgentListBox(this);

		// Get a panel to place the 2 list boxes
		JPanel panel1=new JPanel(new GridLayout(1,3));
		panel1.setBorder(BorderFactory.createEtchedBorder(Color.black,Color.yellow));

		// add the list boxes to the panel
		panel1.add(readyList);
		panel1.add(incomingList);
		panel1.add(resultList);

		// add the panel to the Frame
		c.add(panel1,BorderLayout.CENTER);
	}


	/* add the message window where all the messages will be displayed	*/
	private void addMessageWindow()
	{
		JPanel windowPanel=new JPanel(new BorderLayout());
		windowPanel.setPreferredSize(new Dimension(500,100));
		windowPanel.setBorder(BorderFactory.createEtchedBorder(Color.black,Color.yellow));

		doc = new DefaultStyledDocument();

		JTextPane pane = new JTextPane (doc);
		pane.setSize(500,100);
		pane.setBorder(BorderFactory.createTitledBorder("Message Window"));

		bigStyle = new SimpleAttributeSet();
        StyleConstants.setFontSize(bigStyle, 16);

		windowPanel.add(pane,BorderLayout.CENTER);
		c.add(windowPanel,BorderLayout.SOUTH);

	}

	/* interface method implementation for ActionListener	*/
	public void actionPerformed(ActionEvent evt)
	{
		String command=evt.getActionCommand();

		

		if (command.equals("Spawn"))
		{
			// Create a list of agents
			// Get a user selection
			// Call the agent Spawner to spawn the agent depending on what the user selected
			if (agentSelectedToSpawn==1)
			{
				MyAgent myAgent=new MyAgent();
				// Call the agent Initializer to initialize all the data members of the agent
				agentInitializer=new AgentInitializer(myAgent);
				readyList.addEntry("MyAgent",myAgent);
				selectedAgentName=new String("MyAgent");
				showErrorMessage(selectedAgentName+" spawned and ready for dispatch...");

				
			}

			else if (agentSelectedToSpawn==2)
			{
				
				// Set the values for this agent
				Vector listOfHosts=new Vector();
				try
				{
					listOfHosts.addElement(GlobalValues.bankIPAddress);
					listOfHosts.addElement(GlobalValues.telephoneCompanyIPAddress);
					listOfHosts.addElement(GlobalValues.myIPAddress);
				}
				catch (ArrayIndexOutOfBoundsException e)
				{
					System.out.println("Array Index Out of Bounds Exception");
				}

				InfoAgent infoAgent=new InfoAgent("InfoAgent", GlobalValues.agentOrganization, GlobalValues.myIPAddress, listOfHosts, GlobalValues.bankAccountNumber, GlobalValues.telecomAccountNumber, GlobalValues.lname, GlobalValues.fname);
				// Copy this vector to the agent body
				infoAgent.agentOriginLocation=GlobalValues.myIPAddress;
				readyList.addEntry("InfoAgent",infoAgent);
				selectedAgentName=new String("InfoAgent");
				showErrorMessage(selectedAgentName+" spawned and ready for dispatch...");
		
			}

			else if (agentSelectedToSpawn==3)
			{
					// Set the values for this agent
				Vector listOfHosts=new Vector();
				try
				{
					listOfHosts.addElement(GlobalValues.bankIPAddress);
					listOfHosts.addElement(GlobalValues.myIPAddress);
				}
				catch (ArrayIndexOutOfBoundsException e)
				{
					System.out.println("Array Index Out of Bounds Exception");
				}

				double amount;
				 if (GlobalValues.bankNeeds!=0)
					amount=GlobalValues.bankNeeds;
				else if (GlobalValues.transferAmount!=0)
					amount=GlobalValues.transferAmount;
				else amount=GlobalValues.transferAmount;


				TransferAgent tranAgent=new TransferAgent("TransferAgent", GlobalValues.agentOrganization, GlobalValues.myIPAddress, listOfHosts, GlobalValues.bankAccountNumber,  GlobalValues.lname, GlobalValues.fname, amount);
				// Copy this vector to the agent body
				tranAgent.agentOriginLocation=GlobalValues.myIPAddress;
				readyList.addEntry("TransferAgent",tranAgent);
				selectedAgentName=new String("TransferAgent");
				showErrorMessage(selectedAgentName+" spawned and ready for dispatch...\nTransfer Amount:"+amount);
			}

			else if (agentSelectedToSpawn==4)
			{
				// Set the values for this agent
				Vector listOfHosts=new Vector();
				try
				{
					listOfHosts.addElement(GlobalValues.myIPAddress);
					listOfHosts.addElement(GlobalValues.telephoneCompanyIPAddress);
					
				}
				catch (ArrayIndexOutOfBoundsException e)
				{
					System.out.println("Array Index Out of Bounds Exception");
				}

				BillAgent billAgent=new BillAgent("InfoAgent", GlobalValues.agentOrganization, GlobalValues.myIPAddress, listOfHosts, "108-89-7890", GlobalValues.telecomAccountNumber, GlobalValues.lname, GlobalValues.fname, "201 Maple Avenue", 178.00);
				// Copy this vector to the agent body
				billAgent.agentOriginLocation=GlobalValues.myIPAddress;
				readyList.addEntry("BillAgent",billAgent);
				selectedAgentName=new String("BillAgent");
				showErrorMessage(selectedAgentName+" spawned and ready for dispatch...");
			}

		}

		else if (command.equals("Dispatch"))
		{
			// Get the user selection from the list of ready agents
			int indexOfSelection=readyList.getSelectedAgentIndex();

			if (indexOfSelection==-1)
			{
				showErrorMessage("No agent has been selected");
			}
			else if (indexOfSelection==-2)
			{
				showErrorMessage("List is currently empty");
			}

			// Get the user selected agent
			toDispatch=readyList.getSelectedAgent(indexOfSelection);

			// get the next host address from the agent itself
			String nextHostAddress=toDispatch.getNextHostAddressForDispatch();
			
			// create an Agent Identity for this agent
			AgentIdentity identity=createAgentIdentity();

			// call the agent spawner to dispatch the agent
			AgentSpawner sender=new AgentSpawner(nextHostAddress,selectedAgentName+".java", identity,toDispatch,this);

			// dispatch agent
			ErrorObject message=sender.sendAgent();
			showErrorMessage("Agent dispatched to next destination...");
			if (message.errorCode==-1)
			{
				System.out.println(message.errorMessage);
				displayErrorDialog(message.errorMessage);
				return;
			}

			// remove the agent name from the ready agent list
			readyList.removeAgent(indexOfSelection);
			agentIsNotSelected();

		}

		else if (command.equals("Kill"))
		{
			// remove an instance of the agent from the list
			if (isAgentSelected)
			{
				int indexOfSelection=readyList.getSelectedAgentIndex();
				toDispatch=null;
				readyList.removeAgent(indexOfSelection);
				agentIsNotSelected();
				showErrorMessage("Agent killed and removed from ready list...");
			}
		}

		else if (command.equals("MyAgent"))
		{
			// User has selected MyAgent to be spawned
			agentSelectedToSpawn=1;
		}

		else if (command.equals("InfoAgent"))
		{
			// User has selected InfoAgent to be spawned
			agentSelectedToSpawn=2;
		}

		else if (command.equals("TransferAgent"))
		{
			// User has selected TransferAgent to be spawned
			agentSelectedToSpawn=3;
		}

		else if (command.equals("BillAgent"))
		{
			// Bank Agent
			agentSelectedToSpawn=4;
		}

		else if (command.equals("Start"))
		{
			// Connect the Agent Server
			agentServer=new AgentServer(this,askBeforeDispatch,showReceiveEvents);
		}

		else if (command.equals("Stop"))
		{
		}

		else if (command.equals("ChangeSetting"))
		{
			// display the ChangeSettings Form for the user to enter new values
			displayChangeForm();
		}

		else if (command.equals("showReceiverEvents"))
		{
			// Toggle showReceiverEvents switch on/off
			if (showReceiveEvents) 
			{
				setShowReceiveEvents(false);
				showErrorMessage("Show Receiver Events disabled...");
			}
			else if (!(showReceiveEvents)) 
			{
				setShowReceiveEvents(true);
				showErrorMessage("Show Receiver Events enabled...");
			}
		}

		else if (command.equals("askBeforeDispatch"))
		{
			// Toggle askBeforeDispatch switch on/off
			if (askBeforeDispatch) 
			{
				setAskBeforeDispatch(false);
				showErrorMessage("Ask Before Dispatch disabled...");
			}
			else if (!(askBeforeDispatch)) 
			{
				setAskBeforeDispatch(true);
				showErrorMessage("Ask Before Dispatch enabled...");
			}
		}

		else if (command.equals("askBeforeAccepting"))
		{
			// Toggle askBeforeAccepting switch on/off
			if (getAskBeforeAccepting()) 
			{
				setAskBeforeAccepting(false);
				showErrorMessage("Ask Before Accepting disabled...");
			}
			else if (!(getAskBeforeAccepting())) 
			{
				setAskBeforeAccepting(true);
				showErrorMessage("Ask Before Accepting enabled...");
			}
		}

		else if (command.equals("Forward"))
		{
				displayChangeForm();
		}


		
	}


	public void agentIsSelected()
	{
		// If agent is selected then enable all buttons
		isAgentSelected=true;
		b2.setEnabled(true);
		b3.setEnabled(true);
		b4.setEnabled(true);
	}

	public void agentIsNotSelected()
	{
		// If agent is not selected the disable all buttons
		isAgentSelected=false;
		b2.setEnabled(false);
		b3.setEnabled(false);
		b4.setEnabled(false);
	}

	private void displayChangeForm()
	{
		// Display the ChangeSettings form
		Form input = new Form();
		System.out.println(GlobalValues.address);
	}

	private AgentIdentity createAgentIdentity()
	{
		// Create a new agent identity for the out going agent
		AgentIdentity identity;
		identity=new AgentIdentity();
		identity.agentOrganization=GlobalValues.agentOrganization;
		return identity;
	}

	private   void displayErrorDialog(String errorMessage)
	{
		// Display the dialog box for error messages
		ErrorDialog messageWindow=new ErrorDialog(this,errorMessage,400,100,false);
	}

	public synchronized void showErrorMessage(String message)
	{
		// display the incoming message in the message box
		try
		{
			doc.insertString (0, "                                                    \n                                                \n                                 " , bigStyle);
			doc.insertString (0, message , bigStyle);
		}
		catch (BadLocationException e)
		{}
	}

	public  void  addAgentName(String name)
	{
		// add agent name to the IncomingAgent list
		incomingList.addAgentName(name);
	}

	public  void removeAgentName(String name)
	{
		// remove agent name from the IncomingAgentList
		incomingList.removeAgentName(name);
	}
		
	public void addResultAgentName(String name, Agent agent)
	{
		// add agent name to the Results list
		resultList.addEntry(name,agent);
	}

	public  void showAcceptMessage(String message)
	{
		// remove agent name from the Results list
		new ErrorDialog(this, message, 400,300,true);
	}

	public  boolean getShowReceiveEvents()
	{
		return showReceiveEvents;
	}

	public void setShowReceiveEvents(boolean value)
	{
		showReceiveEvents=value;
	}

	public  boolean getAskBeforeDispatch()
	{
		return askBeforeDispatch;
	}

	public  void setAskBeforeDispatch(boolean value)
	{
		askBeforeDispatch=value;
	}

	public  boolean getAcceptAgent()
	{
		return acceptAgent;
	}

	public void setAcceptAgent(boolean value)
	{
		acceptAgent=value;
	}

	public  boolean getAskBeforeAccepting()
	{
		return askBeforeAccepting;
	}

	public  void setAskBeforeAccepting(boolean value)
	{
		askBeforeAccepting=value;
	}
// change

	public static void main(String a[])
	{
		if (a[0].equals(null) || a[0].equals(""))
		{
			System.out.println("Usage: java AgentServerGUI <user/telecom>");
			System.exit(1);
		}
		new AgentServerGUI(a[0]);
	}

}

		



