/**
Class BillAgent:
One of the agents.
Notifies the user of his bank balance graphically
*/

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.swing.*;
import java.awt.swing.event.*;
import java.security.*;


public class BillAgent extends Agent implements Serializable
{
	String agentName;
	String agentOrganization;
	String agentPreviousLocation;
	String agentOriginLocation;
	Vector agentHostList;

	//Required parameters for the Bill agent
	String socialSecurityNumber;
	String accountNumber;
	String lastName;
	String firstName;
	String address;
	double balance;

	// Security objects
	byte messageDigest[]=null;
	PublicKey publicKey=null;
	byte signature[]=null;

	// constructor
	public BillAgent()
	{
	}

	public BillAgent(String agentName, String org, String ownName, Vector list, String ssn, String accountNumber, String lastName, String firstName, String address, double bal)
	{
		
		this.agentName=agentName;
		this.agentOrganization=org;
		this.agentPreviousLocation=ownName;
		this.agentHostList=list;
		this.agentOriginLocation=ownName;

		this.socialSecurityNumber = ssn;
		this.accountNumber = accountNumber;
		this.lastName = lastName;
		this.firstName = firstName;
		this.address=address;
		this.balance = bal;
	}


	public void tellData()
	{
		System.out.println("Agent Data:\n"+agentName+"\n"+agentOrganization+"\n"+agentPreviousLocation+"\n"+agentOriginLocation+"\n");
	}

	public String getAgentName() 
	{
		return agentName;
	}

	public String getAgentOrganization()
	{
		return agentOrganization;
	}

	public String getAgentOrigin()
	{
		return agentOriginLocation;
	}

	public String getAgentPreviousLocation()
	{
		return agentPreviousLocation;
	}

	public String getNextHostAddressForDispatch()
	{
		String next=null;

		if (agentHostList.isEmpty())
		{
			// If no more destinations then return the origin location for the agent
			return null;
		}

		try
		{
			next=(String) agentHostList.firstElement();
			agentHostList.removeElementAt(0);
		}
		catch (NoSuchElementException e)
		{
			System.out.println("MyAgent ERROR: No such element");
			return null;
		}
		catch (ArrayIndexOutOfBoundsException e1)
		{
			System.out.println("My Agent ERROR:Array Index Out of bounds");
			return null;
		}

		return next;
	}

	public String getNextHostAddress()
	{
		String next=null;

		if (agentHostList.isEmpty())
		{
			// If no more destinations then return the origin location for the agent
			return agentOriginLocation;
		}

		try
		{
			next=(String) agentHostList.firstElement();
		}
		catch (NoSuchElementException e)
		{
			System.out.println("MyAgent ERROR: No such element");
			return null;
		}
		catch (ArrayIndexOutOfBoundsException e1)
		{
			System.out.println("My Agent ERROR:Array Index Out of bounds");
			return null;
		}

		return next;
	}

	public void showResults()
	{
		Double bal = new Double(balance);
		String newbal = bal.toString();

		String colNames[] = {"Social Security No.", "Account No.", "Last Name", "First Name", " Address", "Balance"};
		String row[][] = {
								{"Social Security No.", "Account No.", "Last Name", "First Name", " Address", "BankBal."},
								{" "," "," "," "," "," "},
								{socialSecurityNumber, accountNumber, lastName, firstName, address, newbal}
							
								};
			
		JPanel myPanel=new JPanel();
		myPanel.setLayout(new GridLayout(3,1));
		JTable table=new JTable(row,colNames);

		String msg = "Phone Company's Agent reporting";
		Font font = new Font("Serif", Font.BOLD , 22); 
		ImageIcon bankIcon = new ImageIcon("spawnon.gif"); 
		JLabel label=new JLabel(msg); 
		label.setIcon(bankIcon);
		label.setFont(font);
        label.setHorizontalAlignment(JLabel.LEFT); 

		String msg1 = "Informed " + lastName + " of $" +balance +" balance";
		Font font1 = new Font("Serif", Font.ITALIC , 18); 
		JLabel label1=new JLabel(msg1); 
		label1.setFont(font1);
        label1.setHorizontalAlignment(JLabel.LEFT); 

		myPanel.add(label);
		myPanel.add(table);
		myPanel.add(label1);

		JFrame bankFrame = new JFrame(msg);
		Container c = bankFrame.getContentPane();
		c.setLayout(new FlowLayout());

		bankFrame.setSize(new Dimension(500,200));
		bankFrame.setVisible(true);

		c.add(myPanel);
		bankFrame.show();

	}

	public void tellLocation()
	{}

	public void run() {
		AmountChanger change = new AmountChanger();
		change.changeAmount(balance);

		Double bal = new Double(balance);
		String newbal = bal.toString();

		String colNames[] = {"Social Security No.", "Account No.", "Last Name", "First Name", " Address", "Balance"};
		String row[][] = {
								{"Social Security No.", "Account No.", "Last Name", "First Name", " Address", "BankBal."},
								{" "," "," "," "," "," "},
								{socialSecurityNumber, accountNumber, lastName, firstName, address, newbal}
							
								};
			
		JPanel myPanel=new JPanel();
		myPanel.setLayout(new GridLayout(3,1));
		JTable table=new JTable(row,colNames);

		String msg = "Phone Company's Agent reporting";
		Font font = new Font("Serif", Font.BOLD , 22); 
		ImageIcon bankIcon = new ImageIcon("spawnon.gif"); 
		JLabel label=new JLabel(msg); 
		label.setIcon(bankIcon);
		label.setFont(font);
        label.setHorizontalAlignment(JLabel.LEFT); 

		String msg1 = "Your bill is $" + balance;
		Font font1 = new Font("Serif", Font.ITALIC , 18); 
		JLabel label1=new JLabel(msg1); 
		label1.setFont(font1);
        label1.setHorizontalAlignment(JLabel.LEFT); 

		myPanel.add(label);
		myPanel.add(table);
		myPanel.add(label1);

		JFrame bankFrame = new JFrame(msg);
		Container c = bankFrame.getContentPane();
		c.setLayout(new FlowLayout());

		bankFrame.setSize(new Dimension(500,200));
		bankFrame.setVisible(true);

		c.add(myPanel);
		bankFrame.show();

	}

	public byte[] giveMessageDigest()
	{
		return messageDigest;
	}

	public PublicKey givePublicKey()
	{
		return publicKey;
	}

	public byte[] giveSignature()
	{
		return signature;
	}

	public void signAgent(PublicKey key, byte digest[], byte sign[])
	{
		this.messageDigest=digest;
		this.publicKey=key;
		this.signature=sign;
	}
}

