/**
Class IncomingAgentMonitor:
Listens to incoming requests for agent transfer and spawns a new RequestServicer object for each incoming agent
*/
import java.io.*;
import java.net.*;
import java.util.*;

public class IncomingAgentMonitor extends Thread implements AgentEventListener
{
	ServerSocket serverSocket=null;
	Socket clientSocket=null;
	RequestServicer requestServicer;
	AgentServerGUI gui;
	int numberOfIncomingRequests;
	AgentProgress progress;

	// Settings
	boolean showReceiveEvents=false;
	boolean askBeforeDispatch=false;


	// Constructor
	public IncomingAgentMonitor(AgentServerGUI gui, boolean showEvents, boolean askBeforeDispatch)
	{
		this.showReceiveEvents=showEvents;
		this.askBeforeDispatch=askBeforeDispatch;
		this.gui=gui;
		numberOfIncomingRequests=0;

	}

	public void decrementThreadCount()
	{
		numberOfIncomingRequests--;
	}



	public void run()
	{
		
		try
		{
			// Bind to port 5100 and accept a maximum of 10 incoming connections
			serverSocket=new ServerSocket(5100,10);
		}
		catch (IOException e)
		{
			gui.showErrorMessage("ERROR: Server cannot bind...");
			this.stop();
		}

		gui.showErrorMessage("Incoming Agent Monitor setup complete...");


		RequestServicer req=null;
		// Wait to accept the incoming connection
		while (numberOfIncomingRequests<10)
		{
			try
			{
				// wait and accept incoming requests
				clientSocket=serverSocket.accept();
				gui.showErrorMessage("New Incoming request accepted...");

				// start a new thread to service this connection
				req=new RequestServicer(clientSocket,this,gui,askBeforeDispatch);
				req.addAgentEventListener(this);

				req.start();

			}

			catch (TooManyListenersException e)
			{}
			catch (IOException e)
			{
				gui.showErrorMessage("Server Error: Cannot accept incoming connection...");
			}
		}
	}

	public void agentIdentityArrived(AgentEvent evt)
	{
		if (gui.getShowReceiveEvents())
		{
			progress=new AgentProgress();
			progress.agentIdentityArrived();
		}
	}

	// Agent is authenticated
	public void agentAuthenticated(AgentEvent evt)
	{
		if (gui.getAskBeforeAccepting())
		{
			if (!(evt.message.equals(null))  && !(evt.message.equals("")))
			gui.showAcceptMessage(evt.message);
		}
		if (gui.getShowReceiveEvents())
		{
			progress.agentAuthenticated();
		}

	}

	// Agent arrival
	public void agentArrived(AgentEvent evt)
	{
		if (gui.getShowReceiveEvents())
		{
			progress.agentArrived();
		}
	}

	// Agent "start" method is called to start the agent thread
	public void agentStarted(AgentEvent evt)
	{
		if (gui.getShowReceiveEvents())
		{
			progress.agentStarted();
		}
	}

	public void agentStopped(AgentEvent evt)
	{
	}

	// Agent Authentication failed
	public void agentAuthenticationFailed(AgentEvent evt)
	{
		if (gui.getShowReceiveEvents())
		{
			progress.agentAuthenticationFailed();
		}
	}

}
