/**
class : OutAgentListBox
Class which handles the display and insertion/deletion of the names in the list boxes for the incoming and ready agents
*/
import java.awt.swing.*;
import java.awt.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.swing.border.*;
import java.util.*;


public class OutAgentListBox extends JList implements MouseListener
{

	private DefaultListModel dmodel;
	Vector readyAgentList;
	int selectionIndex=-1;
	AgentServerGUI gui;

	// constructor
	public OutAgentListBox(AgentServerGUI gui)
	{
		super();
		this.gui=gui;
		dmodel=new DefaultListModel();
		setModel(dmodel);
		readyAgentList=new Vector();
		addMouseListener(this);


		setBorder(BorderFactory.createTitledBorder("Ready Agents"));
	}

	public void addEntry(String agentName, Agent agent)
	{
		// Get the size of the list
		// Get the index where the element is going to be added
		// Create a new OutListElement instance
		// Add it to the vector of ready agents
		int listSize=dmodel.getSize();
		dmodel.addElement(agentName);
		OutListElement one=new OutListElement(listSize,agentName,agent);
		addToVector(one);

	}

	private void addToVector(OutListElement element)
	{
		try
		{
			readyAgentList.add(element.listIndexNumber, element);
		}
		catch (ArrayIndexOutOfBoundsException e)
		{}
	}

	public int getSelectedAgentIndex()
	{
		if (selectionIndex!=-1)
			// return the selection index
			return selectionIndex;
		else if (dmodel.getSize()==0)
			// return -2 if the list is empty
			return -2;

		// return -1 for an invalid selection 
		return -1;
	}

	public Agent getSelectedAgent(int index)
	{
		OutListElement element=null;

		try
		{
			element=(OutListElement) readyAgentList.elementAt(index);
		}
		catch (ArrayIndexOutOfBoundsException e)
		{
			System.out.println("Array out of bounds exception caught");
		}

		return element.agent;
	}

	public void removeAgent(int index)
	{
		try
		{
			readyAgentList.removeElementAt(index);
			dmodel.removeElementAt(index);
		}
		catch (ArrayIndexOutOfBoundsException e)
		{
			System.out.println("Array Index Out of Bounds Exception caught");
		}
	}

	public void mouseClicked(MouseEvent evt)
	{
		int index = locationToIndex(evt.getPoint());
		if (index!=-1 && (evt.getClickCount()==2))
		{
			
			selectionIndex=index;
			gui.agentIsSelected();
			try
			{
				gui.selectedAgentName=(String) dmodel.getElementAt(index);
			}
			catch (ArrayIndexOutOfBoundsException e)
			{}
			gui.showErrorMessage(gui.selectedAgentName+" selected for dispatch...");
		}

	
	}

	public void mouseEntered(MouseEvent evt)
	{
	}

	public void mousePressed(MouseEvent evt)
	{
	}

	public void mouseReleased(MouseEvent evt)
	{
	}

	public void mouseExited(MouseEvent evt)
	{
	} 
}
	
