/**
Class: TestSig
	   The class handles the requiste authentication activities.
  */

import java.io.*;
import java.security.*;

class TestSig {

	public TestSig() {
	}

	/*generateSig is used to generate the public key, message digest
		and the signed message digest. This is called by the sender.
		*/

    public ReturnSig generateSig(byte[] data) {
		ReturnSig gBack=null;

        try {
            /* generate a key pair */
            KeyPairGenerator keyGen = KeyPairGenerator.getInstance("DSA");
            keyGen.initialize(1024, new SecureRandom());
            KeyPair pair = keyGen.generateKeyPair();

            /* create objects to use for signing and verifying */
            Signature dsa = Signature.getInstance("SHA/DSA"); //Sign digest
			MessageDigest sha = MessageDigest.getInstance("SHA-1"); //Sender's Digest

            /* initialize the Signature object for signing */
            PrivateKey priv = pair.getPrivate();
            dsa.initSign(priv);

            /* Update and sign the data */
			String y = new String(data);
			int l = y.length();

            byte b;
			int j = 0;
            while (j<l) {
                b = data[j];
				sha.update(b);
				j++;
            };


			System.out.println("Text length: " + j);
			byte[] hash = sha.digest(); //Generate digest

			String x = new String(hash);
			int len = x.length();
			System.err.println("Message Digest length: " + len);
			for(int i = 0; i< len ; i++) {
				dsa.update(hash[i]);
			}

            byte[] sig = dsa.sign(); //Signed digest
			PublicKey pub = pair.getPublic();

			gBack = new ReturnSig();
			gBack.digest = hash;
			gBack.encryptedDigest = sig;
			gBack.pub = pub;
			
		} catch (Exception e) {
            System.err.println("Caught exception " + e.toString());
        }
		return gBack;
	}

	/*verifySig is used to make sure that the recievd digest,
	signed digest and text can are relly authentic. This is called by the receiver.
	*/
	public boolean verifySig(byte[] hash, byte[] ehash, PublicKey pub, byte[] data) {

		boolean ok = false;
		byte b;

        /* Verify the signature */
        /* Initialize the Signature object for verification */
		try {

            /* create objects to use for signing and verifying */
            Signature dsa = Signature.getInstance("SHA/DSA"); //Sign digest
			MessageDigest sha1 = MessageDigest.getInstance("SHA-1"); // Receiver's Digest
            dsa.initVerify(pub);
            /* Update and verify the data */

			String y = new String(data);
			int l = y.length();
			int j = 0;

            while (j<l) {
                b = data[j];
				sha1.update(b);
				j++;
			};

			byte[] hash1 = sha1.digest(); //Generate digest from text
			String x = new String(hash1);
			int len = x.length();

			for(int i = 0; i< len ; i++) {
				dsa.update(hash[i]);
			}

			//Compare digests
			boolean verify = true;
			for(int i = 0; i< len ; i++) 
			if (hash[i] != hash1[i]) {
					verify = false;
					break;
			}

            boolean verifies = dsa.verify(ehash);

			if ((verify) && (verifies)) ok = true;

            System.out.println("Digest signature verifies: " + verifies);
			System.out.println("We generated the same digest: " + verify);

        } catch (Exception e) {
            System.err.println("Caught exception " + e.toString());
        }

			return ok;
    }

}
